@@ -33,6 +33,7 @@ class TestServer(unittest.TestCase):
     _mock_async._async = {
         'thread': 't',
         'queue': 'q',
+        'queue_empty': RuntimeError,
         'websocket': 'w',
     }
 
@@ -116,8 +117,10 @@ class TestServer(unittest.TestCase):
     @mock.patch('importlib.import_module', side_effect=_mock_import)
     def test_async_mode_gevent_uwsgi(self, import_module):
         sys.modules['gevent'] = mock.MagicMock()
-        sys.modules['gevent.queue'] = mock.MagicMock()
+        sys.modules['gevent'].queue = mock.MagicMock()
+        sys.modules['gevent.queue'] = sys.modules['gevent'].queue
         sys.modules['gevent.queue'].JoinableQueue = 'foo'
+        sys.modules['gevent.queue'].Empty = RuntimeError
         sys.modules['gevent.event'] = mock.MagicMock()
         sys.modules['gevent.event'].Event = 'bar'
         sys.modules['uwsgi'] = mock.MagicMock()
@@ -128,6 +131,7 @@ class TestServer(unittest.TestCase):
 
         self.assertEqual(s._async['thread'], async_gevent_uwsgi.Thread)
         self.assertEqual(s._async['queue'], 'foo')
+        self.assertEqual(s._async['queue_empty'], RuntimeError)
         self.assertEqual(s._async['event'], 'bar')
         self.assertEqual(s._async['websocket'],
                          async_gevent_uwsgi.uWSGIWebSocket)
@@ -140,8 +144,10 @@ class TestServer(unittest.TestCase):
     @mock.patch('importlib.import_module', side_effect=_mock_import)
     def test_async_mode_gevent_uwsgi_without_uwsgi(self, import_module):
         sys.modules['gevent'] = mock.MagicMock()
-        sys.modules['gevent.queue'] = mock.MagicMock()
+        sys.modules['gevent'].queue = mock.MagicMock()
+        sys.modules['gevent.queue'] = sys.modules['gevent'].queue
         sys.modules['gevent.queue'].JoinableQueue = 'foo'
+        sys.modules['gevent.queue'].Empty = RuntimeError
         sys.modules['gevent.event'] = mock.MagicMock()
         sys.modules['gevent.event'].Event = 'bar'
         sys.modules['uwsgi'] = None
@@ -155,8 +161,10 @@ class TestServer(unittest.TestCase):
     @mock.patch('importlib.import_module', side_effect=_mock_import)
     def test_async_mode_gevent_uwsgi_without_websocket(self, import_module):
         sys.modules['gevent'] = mock.MagicMock()
-        sys.modules['gevent.queue'] = mock.MagicMock()
+        sys.modules['gevent'].queue = mock.MagicMock()
+        sys.modules['gevent.queue'] = sys.modules['gevent'].queue
         sys.modules['gevent.queue'].JoinableQueue = 'foo'
+        sys.modules['gevent.queue'].Empty = RuntimeError
         sys.modules['gevent.event'] = mock.MagicMock()
         sys.modules['gevent.event'].Event = 'bar'
         sys.modules['uwsgi'] = mock.MagicMock()
@@ -168,6 +176,7 @@ class TestServer(unittest.TestCase):
 
         self.assertEqual(s._async['thread'], async_gevent_uwsgi.Thread)
         self.assertEqual(s._async['queue'], 'foo')
+        self.assertEqual(s._async['queue_empty'], RuntimeError)
         self.assertEqual(s._async['event'], 'bar')
         self.assertEqual(s._async['websocket'], None)
         del sys.modules['gevent']
@@ -179,8 +188,10 @@ class TestServer(unittest.TestCase):
     @mock.patch('importlib.import_module', side_effect=_mock_import)
     def test_async_mode_gevent(self, import_module):
         sys.modules['gevent'] = mock.MagicMock()
-        sys.modules['gevent.queue'] = mock.MagicMock()
+        sys.modules['gevent'].queue = mock.MagicMock()
+        sys.modules['gevent.queue'] = sys.modules['gevent'].queue
         sys.modules['gevent.queue'].JoinableQueue = 'foo'
+        sys.modules['gevent.queue'].Empty = RuntimeError
         sys.modules['gevent.event'] = mock.MagicMock()
         sys.modules['gevent.event'].Event = 'bar'
         sys.modules['geventwebsocket'] = 'geventwebsocket'
@@ -191,6 +202,7 @@ class TestServer(unittest.TestCase):
 
         self.assertEqual(s._async['thread'], async_gevent.Thread)
         self.assertEqual(s._async['queue'], 'foo')
+        self.assertEqual(s._async['queue_empty'], RuntimeError)
         self.assertEqual(s._async['event'], 'bar')
         self.assertEqual(s._async['websocket'], async_gevent.WebSocketWSGI)
         del sys.modules['gevent']
@@ -202,8 +214,10 @@ class TestServer(unittest.TestCase):
     @mock.patch('importlib.import_module', side_effect=_mock_import)
     def test_async_mode_gevent_without_websocket(self, import_module):
         sys.modules['gevent'] = mock.MagicMock()
-        sys.modules['gevent.queue'] = mock.MagicMock()
+        sys.modules['gevent'].queue = mock.MagicMock()
+        sys.modules['gevent.queue'] = sys.modules['gevent'].queue
         sys.modules['gevent.queue'].JoinableQueue = 'foo'
+        sys.modules['gevent.queue'].Empty = RuntimeError
         sys.modules['gevent.event'] = mock.MagicMock()
         sys.modules['gevent.event'].Event = 'bar'
         sys.modules['geventwebsocket'] = None
@@ -214,6 +228,7 @@ class TestServer(unittest.TestCase):
 
         self.assertEqual(s._async['thread'], async_gevent.Thread)
         self.assertEqual(s._async['queue'], 'foo')
+        self.assertEqual(s._async['queue_empty'], RuntimeError)
         self.assertEqual(s._async['event'], 'bar')
         self.assertEqual(s._async['websocket'], None)
         del sys.modules['gevent']
@@ -375,7 +390,7 @@ class TestServer(unittest.TestCase):
         s.sockets['foo'].closed = True
         self.assertRaises(KeyError, s._get_socket, 'foo')
 
-    def test_jsonp_not_supported(self):
+    def test_jsonp_with_bad_index(self):
         s = server.Server()
         environ = {'REQUEST_METHOD': 'GET', 'QUERY_STRING': 'j=abc'}
         start_response = mock.MagicMock()
@@ -383,6 +398,16 @@ class TestServer(unittest.TestCase):
         self.assertEqual(start_response.call_args[0][0],
                          '400 BAD REQUEST')
 
+    def test_jsonp_index(self):
+        s = server.Server()
+        environ = {'REQUEST_METHOD': 'GET', 'QUERY_STRING': 'j=233'}
+        start_response = mock.MagicMock()
+        r = s.handle_request(environ, start_response)
+        self.assertEqual(start_response.call_args[0][0],
+                         '200 OK')
+        self.assertTrue(r[0].startswith(b'___eio[233]("'))
+        self.assertTrue(r[0].endswith(b'");'))
+
     def test_connect(self):
         s = server.Server()
         environ = {'REQUEST_METHOD': 'GET', 'QUERY_STRING': ''}
@@ -908,7 +933,8 @@ class TestServer(unittest.TestCase):
     def test_create_queue(self):
         s = server.Server()
         q = s.create_queue()
-        self.assertRaises(q.Empty, q.get, timeout=0.01)
+        empty = s.get_queue_empty_exception()
+        self.assertRaises(empty, q.get, timeout=0.01)
 
     def test_create_event(self):
         s = server.Server()
